@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2021, PyInstaller Development Team.
+# Copyright (c) 2021-2022, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -8,8 +8,8 @@
 #
 # SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
 #-----------------------------------------------------------------------------
-from PyInstaller.utils.hooks.qt import pyqt6_library_info, \
-    add_qt6_dependencies, get_qt_qml_files
+
+from PyInstaller.utils.hooks.qt import add_qt6_dependencies, get_qt_qml_files, pyqt6_library_info
 
 hiddenimports, binaries, datas = add_qt6_dependencies(__file__)
 qml_binaries, qml_datas = get_qt_qml_files(pyqt6_library_info)
